import { world } from "@minecraft/server";
import { ActionFormData } from "@minecraft/server-ui";

const getRandomWeightedString = (strings) => strings.reduce((a, { text, weight }) => a.concat(Array(weight).fill(text)), []).sort(() => Math.random() - 0.5)[0]
// Definiere deine Tags mit Titeltexten, Icon-Pfaden und Commands

const strings = [
    { text: "Skins oder keine Skins", weight: 1 },
    { text: "Codes?", weight: 2 },
    { text: "Olympic Games is living", weight: 1 }
];
const tagCommands = {
    "tagA": {
        tag: "tagA",
        title: "Rabe",
        iconPath: "textures/items/stick.png",
        commands: ["say tagA"]
    },
    "tagB": {
        tag: "tagB",
        title: "König",
        iconPath: "textures/items/apple.png",
        commands: ["say tagB"]
    },
    "tagC": {
        tag: "tagC",
        title: "Jesus",
        iconPath: "textures/ui/anvil_icon.png",
        commands: ["say tagC"]
    }
};

const vMapping = {
    0: "command",
    1: "skins",
    2: "codes"
};

// Listen for item use events (e.g., using a Minecraft apple)
world.afterEvents.itemUse.subscribe((eventData) => {
    const player = eventData.source;
    const item = eventData.itemStack;

    // Check if the item used is a Minecraft apple
    if (item.typeId === "minecraft:apple") {
        // Create an action form for the main menu
        const action = new ActionFormData();

        // Set the form title and body
        action.title("Hallo");
        action.body("Choose an option to proceed:");

        // Add buttons for each available option
        Object.values(vMapping).forEach((option) => {
            action.button(option);
        });

        // Show the main menu to the player
        action.show(player).then(async (tagData) => {
            const optionSelection = vMapping[tagData.selection];

            if (optionSelection === "command") {
                // Create an action form for tag selection
                const tagAction = new ActionFormData();

                // Set the form title and body
                tagAction.title("form_title");
                tagAction.body("Choose a tag to execute commands:");

                // Create an array of available tags based on the player's tags
                const playerTags = Array.from(player.getTags());
                const availableTags = playerTags.filter((tag) => tagCommands.hasOwnProperty(tag));

                //for (const command of playerTags) {
                //  console.error("command: " + command)
                //}
                //for (const command of availableTags) {
                //  console.error("availableTags: " + command)
                //}
                const ownList = {};
                // Add buttons for each available tag with their title text and icon path
                availableTags.forEach((tag, index) => {
                    const tagNumber = index + 1;
                    const titleText = tagCommands[tag].title;
                    const taggiver = tagCommands[tag].tag
                    const iconPath = tagCommands[tag].iconPath;
                    tagAction.button(titleText, iconPath);
                    ownList[taggiver] = index;
                });
                tagAction.button("Close");
                world.sendMessage(ownList.toString());
                // Show the tag selection menu to the player
                tagAction.show(player).then(async (tagData) => {
                    const tagSelection = parseInt(tagData.selection);
                    if (tagData.canceled) {
                        return;
                    }
                    // Überprüfe, ob die Auswahl eine gültige numerische Option ist
                    if (!isNaN(tagSelection) && tagSelection >= 0 && tagSelection < Object.values(ownList).length) {
                        // Konvertiere die numerische Auswahl zurück in den tatsächlichen Tag-Namen
                        const selectedTag = Object.keys(ownList).find(tag => ownList[tag] === tagSelection);
                        //console.error("§5" + ownList.toString())
                        //console.error("§6" + tagMapping)
                        //console.error(selectedTag)
                        //console.error("Ausgabe überprüfen...")

                        //console.error(tagSelection)
                        //console.error("selTag: " + selectedTag)
                        //console.error(tagCommands[selectedTag].title)
                        // Überprüfe, ob der ausgewählte Tag in tagCommands existiert
                        //if (selectedTag !== null && tagCommands[selectedTag]) {
                        const commands = tagCommands[selectedTag].commands;
                        //console.error(commands)
                        if (Array.isArray(commands)) {
                            for (const command of commands) {
                                await player.runCommandAsync(command);
                            }
                        } else {
                            player.sendMessage("No commands defined for this selection.");
                        }
                        console.error(tagSelection)
                        //} else {
                        //    player.sendMessage("Invalid tag selection.");
                        //}
                    } else {
                        const ranom_string = getRandomWeightedString(strings);
                        player.sendMessage(ranom_string.toString());
                    }
                }).catch((error) => {
                    console.error(`Error showing tag action form: ${error}`);
                });
            }
        }).catch((error) => {
            console.error(`Error showing action form: ${error}`);
        });
    }
})


